package com.yunzhi.liyuanhui.common;

import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;

import javax.servlet.http.HttpServletRequest;
import java.security.Key;
import java.util.Calendar;
import java.util.Date;
import java.util.Map;

/**
 * JWTUtils
 * jwt 辅助类
 * https://github.com/jwtk/jjwt
 */
public class JWTUtils {
    // 直接定义常量是不安全的
    public final static Key key = Keys.hmacShaKeyFor("yansenisahero,2019-5-17 16:18:14".getBytes());

    public final static String HeaderName = "x-authorization";
    
    /**
     * getSubject 获取 jwt subject
     * @param request
     * @return
     */
    public static String getSubject(HttpServletRequest request) {
        String jws = getToken(request);
        if (null == jws || "".equals(jws.trim())) return null;

        try {
            return Jwts.parser().setSigningKey(key).parseClaimsJws(jws.trim()).getBody().getSubject();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }
    
    /**
     * getSubject 获取 jwt subject
     * @param request
     * @return
     */
    public static Map getUserIdAndOrgId(HttpServletRequest request) {
        String jws = getToken(request);
        if (null == jws || "".equals(jws.trim())) return null;
        
        try {
            return Jwts.parser().setSigningKey(key).parseClaimsJws(jws.trim()).getBody();
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        return null;
    }
    
    /**
     * getSubject 获取 jwt subject
     * @param request
     * @return
     */
    public static String getToken(HttpServletRequest request) {
        return request.getHeader(HeaderName);
    }

    /**
     * newToken 生成新的 token
     * @param subject
     * @return
     */
    public static String newToken(String subject) {
        // 有效期 7 天
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.DAY_OF_MONTH,calendar.get(Calendar.DAY_OF_MONTH)+7);
        return Jwts.builder().setSubject(subject).setExpiration(calendar.getTime()).signWith(key).compact();
    }
    
    /**
     * newToken 生成新的 token
     * @param subject
     * @return
     */
    public static String newTokenByMap(Map subject) {
        // 有效期 30 分钟
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.MINUTE, 30);

        return Jwts.builder().setClaims(subject).setExpiration(calendar.getTime()).signWith(key).compact();
    }
}
